function x = fixpunkt(phi,x0,maxitr,atol,rtol)

% function x = fixpunkt(phi,x0,maxitr,atol,rtol)
%
% Iteriert die Fixpunktiteration mit Fixpunktfunktion phi mit Startwert x0
% bis auf eine kombinierte absolute atol und relative rtol Toleranz (s. Kap. 4,
% Abbruchkriterium ABK3) oder die maximale Anzahl Iterationen maxitr erreicht
% ist.
%
% Parameters: phi       ... Fixpunktfunktion
%             x0        ... Startwert
%             maxitr    ... Maximal Anzahl Iterationen
%             atol,rtol ... Absolute/Relative Toleranz
%
% Returns: x ... Iterationen x^{(k)}

  x    = x0;
  xold = x0;
  xnew = phi(xold);
  x    = [x,xnew];
  err  = abs(xnew-xold);

  itr = 0;
  while err > atol + rtol*abs(xnew) && itr <= maxitr
    xnew = phi(xold);
    err  = abs(xnew-xold);
    x    = [x,xnew];
    xold = xnew;
    itr = itr+1;
  end
