function x = newton(x0,F,DF,nMax,ATOL,RTOL)

% function x = newton(x0,F,DF,nMax,ATOL,RTOL)
%
% Loese das Nullstellenproblem F(x) = 0 mit dem Newton Verfahren.
%
% Parameters: x0        ... Startwert
%             F,DF      ... Funktion und zugehoerige Jacobi-Matrix
%             nMax      ... Maximal Anzahl Iterationen
%             ATOL,RTOL ... Absolute/Relative Toleranz
%
% Returns: x ... Naehrungsloesung

  % Startwert fuer Newton und Wert von F am Startwert
  x = x0;

  % Newton Iterationen
  for ii = 1:nMax
    % Wert von F und DF in alter Iterierte
    Fx = F(x);
    DFx = DF(x);

    % Newton update
    delta_x = - DFx\Fx;

    % Neue Iterierte
    x = x + delta_x;

    % Wenn update zu klein -> stop and exit
    if norm(delta_x) < ATOL + RTOL*norm(x)
      break
    end
  end
end
