% zu loesende nichtlineare Gleichung
f = @(x) exp(2.*x) - sin(x) - 2;
% deren Ableitung
df = @(x) 2.*exp(2.*x) - cos(x);
% Schaetzung der Ableitung mit finiten Differenzen
h = 1.e-7; % Achtung: Nicht zu klein waehlen, warum?
df_h = @(x) (f(x + h) - f(x))/h;

% ein Plot hilft uns gute Startwerte zu finden!
x = linspace(0., 1.);
plot(x, f(x))
xlabel('x');
ylabel('y');
legend('f(x)');
grid;
% print('skalaregleichung.eps','-depsc');

% mit fsolve
fsolve(f, 0.4)

% mit dem Biskektions-Verfahren (implementiert weiter unten)
bisektion(f, 0., 1., 100)

% mit dem Newton-Verfahren
newton(f, df, 0.4, 1.e-7, 1.e-7, 100)

% mit dem Newton-Verfahren und geschaetzter Ableitung
newton(f, df_h, 0.4, 1.e-7, 1.e-7, 100)

% mit dem Sekanten-Verfahren
sekanten(f, 0., 1., 1.e-7, 1.e-7, 100)

function c = bisektion(f, a, b, Nmax)
  fa = f(a);
  fb = f(b);
  if fa*fb > 0.
    error('Fehler: a und b schliessen keine Nullstelle eine!')
  end
  for iter=1:Nmax
    c = 0.5*(a + b);
    fc = f(c);
    if fa*fc < 0.
      b = c;
      fb = fc;
    else
      a = c;
      fa = fc;
    end
  end
end

function x = newton(f, df, x0, atol, rtol, Nmax)
  x = x0;
  for iter=1:Nmax
    dx = f(x)/df(x);
    x = x - dx;
    if abs(dx) <= atol + rtol*abs(x)
      break;
    end
  end
end

function x = sekanten(f, x0, x1, atol, rtol, Nmax)
  xk  = x1; % x^{(k)}
  fxk = f(xk); % f(x^{(k)})
  xkm = x0; % x^{(k-1)}
  fxkm = f(xkm); % x^{(k-1)}
  for iter=1:Nmax
    dx = (xk - xkm)/(fxk - fxkm)*fxk;
    xkm = xk;
    fxkm = fxk;
    xk = xk - dx;
    if abs(dx) <= atol + rtol*abs(xk)
      break;
    end
    fxk = f(xkm);
  end
end
